@@ -1,27 +1,29 @@
 import asyncio
 import logging
 import json
+import os
 import sys
 from functools import partial
 from contextlib import contextmanager
 
-from fuocore import LiveLyric, Library
-from fuocore.dispatch import Signal
-from fuocore.models import Resolver, reverse, resolve, \
+from feeluown.library import Library
+from feeluown.utils.dispatch import Signal
+from feeluown.models import Resolver, reverse, resolve, \
     ResolverNotFound
-from fuocore.playlist import PlaybackMode
-from fuocore.pubsub import (
+from feeluown.player import PlaybackMode
+
+from feeluown.lyric import LiveLyric
+from feeluown.pubsub import (
     Gateway as PubsubGateway,
     HandlerV1 as PubsubHandlerV1,
+    LiveLyricPublisher
 )
 
-from .consts import APP_ICON, STATE_FILE
-from .fm import FM
-from .player import Player
+from feeluown.utils.request import Request
+from feeluown.rpc.server import FuoServer
+from .consts import STATE_FILE
+from .player import FM, Player
 from .plugin import PluginsManager
-from .server import FuoServer
-from .publishers import LiveLyricPublisher
-from .request import Request
 from .version import VersionManager
 from .task import TaskManager
 
@@ -76,7 +78,7 @@ class App:
                     songs.append(song)
             playlist.init_from(songs)
             if songs and self.mode & App.GuiMode:
-                self.browser.goto(uri='/player_playlist')
+                self.browser.goto(page='/player_playlist')
 
             song = state['song']
 
@@ -153,7 +155,7 @@ def attach_attrs(app):
     """初始化 app 属性"""
     loop = asyncio.get_event_loop()
     app.library = Library(app.config.PROVIDERS_STANDBY)
-    app.live_lyric = LiveLyric()
+    app.live_lyric = LiveLyric(app)
     player_kwargs = dict(
         audio_device=bytes(app.config.MPV_AUDIO_DEVICE, 'utf-8')
     )
@@ -179,13 +181,13 @@ def attach_attrs(app):
         from feeluown.uimodels.collection import CollectionUiManager
         from feeluown.collection import CollectionManager
 
-        from .browser import Browser
-        from .hotkey import HotkeyManager
-        from .image import ImgManager
-        from .theme import ThemeManager
-        from .tips import TipsManager
-        from .ui import Ui
-        from .tray import Tray
+        from .gui.browser import Browser
+        from .gui.hotkey import HotkeyManager
+        from .gui.image import ImgManager
+        from .gui.theme import ThemeManager
+        from .gui.tips import TipsManager
+        from .gui.ui import Ui
+        from .gui.tray import Tray
 
         # GUI 的一些辅助管理模块
         app.coll_mgr = CollectionManager(app)
@@ -212,7 +214,7 @@ def create_app(config):
 
     if mode & App.GuiMode:
 
-        from PyQt5.QtCore import Qt
+        from PyQt5.QtCore import Qt, QDir
         from PyQt5.QtGui import QIcon, QPixmap
         from PyQt5.QtWidgets import QApplication, QWidget
 
@@ -224,9 +226,13 @@ def create_app(config):
         except ImportError:
             logger.info('import QtWebEngineWidgets failed')
 
-        from feeluown.compat import QEventLoop
+        from feeluown.utils.compat import QEventLoop
+
+        pkg_root_dir = os.path.dirname(__file__)
+        icons_dir = os.path.join(pkg_root_dir, 'icons')
 
         q_app = QApplication(sys.argv)
+        QDir.addSearchPath('icons', icons_dir)
 
         q_app.setQuitOnLastWindowClosed(not config.ENABLE_TRAY)
         q_app.setApplicationName('FeelUOwn')
@@ -240,16 +246,18 @@ def create_app(config):
             def __init__(self):
                 super().__init__()
                 self.setObjectName('app')
-                QApplication.setWindowIcon(QIcon(QPixmap(APP_ICON)))
+                QApplication.setWindowIcon(QIcon(QPixmap('icons:feeluown.png')))
 
             def closeEvent(self, _):
                 if not self.config.ENABLE_TRAY:
                     self.exit()
 
             def exit(self):
-                self.ui.mpv_widget.close()
-                event_loop = asyncio.get_event_loop()
-                event_loop.stop()
+                self.about_to_shutdown.emit(app)
+                Signal.teardown_aio_support()
+                self.player.stop()
+                # self.player.shutdown()  # this cause 'abort trap' on macOS
+                QApplication.quit()
 
             def mouseReleaseEvent(self, e):
                 if not self.rect().contains(e.pos()):
@@ -283,6 +291,7 @@ def init_app(app):
 
     app.plugin_mgr.scan()
     if app.mode & App.GuiMode:
+        app.hotkey_mgr.initialize()
         app.theme_mgr.initialize()
         if app.config.ENABLE_TRAY:
             app.tray.initialize()
@@ -326,7 +335,7 @@ def run_app(app):
         # NOTE: gracefully shutdown?
         pass
     finally:
-        _shutdown_app(app)
+        app.exit()
         loop.stop()
         loop.close()
 
@@ -339,13 +348,6 @@ def run_app_once(app, future):
     except KeyboardInterrupt:
         pass
     finally:
-        _shutdown_app(app)
+        app.exit()
         loop.stop()
         loop.close()
-
-
-def _shutdown_app(app):
-    app.about_to_shutdown.emit(app)
-    app.player.stop()
-    app.player.shutdown()
-    Signal.teardown_aio_support()
