@@ -15,12 +15,13 @@ from PyQt5.QtWidgets import (
     QStyle, QSizePolicy, QStyleOptionButton, QStyledItemDelegate,
 )
 
-from fuocore.dispatch import Signal
-from fuocore.excs import ProviderIOError
-from fuocore.models import ModelExistence
+from feeluown.utils.dispatch import Signal
+from feeluown.excs import ProviderIOError
+from feeluown.library import ModelState, ModelFlags
+from feeluown.models import ModelExistence
 
-from feeluown.mimedata import ModelMimeData
-from feeluown.helpers import ItemViewNoScrollMixin
+from feeluown.gui.mimedata import ModelMimeData
+from feeluown.gui.helpers import ItemViewNoScrollMixin, ReaderFetchMoreMixin
 
 
 logger = logging.getLogger(__name__)
@@ -176,67 +177,54 @@ class SongListView(QListView):
         self.play_song_needed.emit(index.data(Qt.UserRole))
 
 
-class SongsTableModel(QAbstractTableModel):
-    def __init__(self, songs=None, source_name_map=None, songs_g=None, parent=None):
+class SongsTableModel(QAbstractTableModel, ReaderFetchMoreMixin):
+    def __init__(self, source_name_map=None, reader=None, parent=None):
         """
 
         :param songs: 歌曲列表
         :param songs_g: 歌曲列表生成器（当歌曲列表生成器不为 None 时，忽略 songs 参数）
         """
         super().__init__(parent)
-        self.songs_g = songs_g
-        self.songs = (songs or []) if self.songs_g is None else []
-        self._can_fetch_more = self.songs_g is not None
+        self._reader = reader
+        self._fetch_more_step = 30
+        self._items = []
+        self._is_fetching = False
+
         self._source_name_map = source_name_map or {}
 
     def removeRows(self, row, count, parent=QModelIndex()):
         self.beginRemoveRows(parent, row, row + count - 1)
         while count > 0:
-            self.songs.pop(row)
+            self._items.pop(row)
             count -= 1
         self.endRemoveRows()
         return True
 
-    def canFetchMore(self, _):
-        return self._can_fetch_more
-
-    def fetchMore(self, _):
-        songs = []
-        current = len(self.songs)
-        fetched = self.songs_g.offset
-        if current < fetched:
-            # FIXME: maybe SequentialReader should support seek
-            songs = self.songs_g._objects[current:fetched]
-        else:
-            for _ in range(current, len(self.songs) + 30):
-                try:
-                    song = next(self.songs_g)
-                except StopIteration:
-                    self._can_fetch_more = False
-                    break
-                except ProviderIOError:
-                    logger.exception("fetch more failed")
-                    break
-                else:
-                    songs.append(song)
-        self.beginInsertRows(QModelIndex(), current, current + len(songs) - 1)
-        self.songs.extend(songs)
-        self.endInsertRows()
-
     def flags(self, index):
+        if index.column() in (Column.source, Column.index, Column.duration):
+            return Qt.NoItemFlags
+
         song = index.data(Qt.UserRole)
         flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable
-        if song and song.exists == ModelExistence.no or \
-           index.column() in (Column.source, Column.index, Column.duration):
-            return Qt.ItemIsSelectable
         if index.column() in (Column.song, Column.album):
-            return flags | Qt.ItemIsDragEnabled
+            flags = flags | Qt.ItemIsDragEnabled
+        if ModelFlags.v2 & song.meta.flags:
+            if song.state is ModelState.not_exists:
+                flags = Qt.NoItemFlags
+            elif song.state is ModelState.cant_upgrade:
+                if index.column() == Column.song:
+                    flags = Qt.ItemIsSelectable | Qt.ItemIsEnabled
+                else:
+                    flags = Qt.NoItemFlags
+        else:
+            if song and song.exists == ModelExistence.no:
+                flags = Qt.NoItemFlags
         return flags
 
     def rowCount(self, parent=QModelIndex()):
-        return len(self.songs)
+        return len(self._items)
 
-    def columnCount(self, _):
+    def columnCount(self, _=QModelIndex()):
         return 6
 
     def headerData(self, section, orientation, role=Qt.DisplayRole):
@@ -246,7 +234,7 @@ class SongsTableModel(QAbstractTableModel):
                 if section < len(sections):
                     return sections[section]
                 return ''
-            elif role == Qt.SizeHintRole:
+            elif role == Qt.SizeHintRole and self.parent() is not None:
                 # we set height to 25 since the header can be short under macOS.
                 # HELP: set height to fixed value manually is not so elegant
                 height = 25
@@ -269,10 +257,10 @@ class SongsTableModel(QAbstractTableModel):
     def data(self, index, role=Qt.DisplayRole):
         if not index.isValid():
             return QVariant()
-        if index.row() >= len(self.songs) or index.row() < 0:
+        if index.row() >= len(self._items) or index.row() < 0:
             return QVariant()
 
-        song = self.songs[index.row()]
+        song = self._items[index.row()]
         if role in (Qt.DisplayRole, Qt.ToolTipRole):
             if index.column() == Column.index:
                 return index.row() + 1
@@ -431,7 +419,7 @@ class SongsTableDelegate(QStyledItemDelegate):
         can be uncertain. I don't know why this would happen,
         since we have set width for the header.
         """
-        if index.isValid():
+        if index.isValid() and self.parent() is not None:
             widths = (0.05, 0.1, 0.25, 0.1, 0.2, 0.3)
             width = self.parent().width()
             w = int(width * widths[index.column()])
@@ -469,7 +457,6 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
 
         self.delegate = SongsTableDelegate(self)
         self.setItemDelegate(self.delegate)
-        self.activated.connect(self._on_activated)
         self.about_to_show_menu = Signal()
 
         self._setup_ui()
@@ -494,34 +481,8 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
         self.setDragEnabled(True)
         self.setDragDropMode(QAbstractItemView.DragOnly)
 
-    def _on_activated(self, index):
-        try:
-            if index.column() == Column.song:
-                song = index.data(Qt.UserRole)
-                self.play_song_needed.emit(song)
-            elif index.column() == Column.artist:
-                song = index.data(Qt.UserRole)
-                artists = song.artists
-                if artists is not None:
-                    if len(artists) > 1:
-                        self.edit(index)
-                    else:
-                        self.show_artist_needed.emit(artists[0])
-            elif index.column() == Column.album:
-                song = index.data(Qt.UserRole)
-                album = song.album
-                self.show_album_needed.emit(album)
-        except (ProviderIOError, Exception):
-            # FIXME: we should only catch ProviderIOError here,
-            # but currently, some plugins such fuo-qqmusic may raise
-            # requests.RequestException
-            logger.exception('fetch song.album failed')
-        # FIXME: 在点击之后，音乐数据可能会有更新，理应触发界面更新
-        # 测试 dataChanged 似乎不能按照预期工作
-        model = self.model()
-        topleft = model.index(index.row(), 0)
-        bottomright = model.index(index.row(), 4)
-        model.dataChanged.emit(topleft, bottomright, [])
+    def show_artists_by_index(self, index):
+        self.edit(index)
 
     def contextMenuEvent(self, event):
         indexes = self.selectionModel().selectedIndexes()
@@ -543,15 +504,18 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
             menu.addSeparator()
             menu.addAction(remove_song_action)
 
+        model = self.model()
+        models = [model.data(index, Qt.UserRole) for index in indexes]
+
         def add_action(text, callback):
             action = QAction(text, menu)
             menu.addSeparator()
             menu.addAction(action)
-            model = self.model()
-            action.triggered.connect(
-                lambda: callback([model.data(index, Qt.UserRole) for index in indexes]))
+            action.triggered.connect(lambda: callback(models))
 
-        self.about_to_show_menu.emit({'add_action': add_action})
+        # .. versionadded: v3.7
+        #   The context key *models*
+        self.about_to_show_menu.emit({'add_action': add_action, 'models': models})
         menu.exec(event.globalPos())
 
     def _add_to_playlist(self, indexes):
@@ -573,9 +537,3 @@ class SongsTableView(ItemViewNoScrollMixin, QTableView):
                 self.remove_song_func(song)
                 distinct_rows.add(row)
         source_model.removeRows(indexes[0].row(), len(distinct_rows))
-
-    def mouseReleaseEvent(self, e):
-        if e.button() in (Qt.BackButton, Qt.ForwardButton):
-            e.ignore()
-        else:
-            super().mouseReleaseEvent(e)
